import Vue from 'vue'
import Cookies from 'js-cookie'

import 'normalize.css/normalize.css' // a modern alternative to CSS resets
import Element from 'element-ui'
import locale from 'element-ui/lib/locale/lang/en'
import './styles/element-variables.scss'

import '@/styles/index.scss' // global css
import App from './App'
import store from './store'
import router from './router'
import VueLazyLoad from 'vue-lazyload';

// 最外层static目录下的图片引用
Vue.use(VueLazyLoad, {
  error: require('./assets/file.png'), // 此处是图片加载失败时候 显示的图片
  // loading: '/static/images/defaultAvatar.png', // 此处是图片加载中 显示的图片
  attempt: 1, // 加载一屏图片
  preLoad: 1, // 失败尝试次数
});
import './icons' // icon
import './permission' // permission control
import './utils/error-log' // error log
import * as filters from './filters' // global filters
import '@/utils/vue-install'
import directive from "@/directive";

Vue.use(Element, {
    size: Cookies.get('size') || 'medium' ,// set element-ui default size
    locale
})

Vue.use(directive)

// register global utility filters
Object.keys(filters).forEach(key => {
    Vue.filter(key, filters[key])
})

Vue.config.productionTip = false


Element.Table.props.height.default = 575

new Vue({
    el: '#app',
    router,
    store,
    render: h => h(App)
})
