import { Loading, Message, MessageBox, Notification } from 'element-ui'

let loadingInstance

export default {
  // 消息提示
  msg(content) {
    Message.info(content)
  },
  // 错误消息
  msgError(content) {
    Message.error(content)
  },
  // 成功消息
  msgSuccess(content) {
    Message.success(content)
  },
  // 警告消息
  msgWarning(content) {
    Message.warning(content)
  },
  // 弹出提示
  alert(content) {
    MessageBox.alert(content, '提示')
  },
  // 错误提示
  alertError(content) {
    MessageBox.alert(content, '提示', { type: 'error' })
  },
  // 成功提示
  alertSuccess(content) {
    MessageBox.alert(content, '提示', { type: 'success' })
  },
  // 警告提示
  alertWarning(content) {
    MessageBox.alert(content, '提示', { type: 'warning' })
  },
  // 通知提示
  notify(content) {
    Notification.info(content)
  },
  // 错误通知
  notifyError(content) {
    Notification.error(content)
  },
  // 成功通知
  notifySuccess(content) {
    Notification.success(content)
  },
  // 警告通知
  notifyWarning(content) {
    Notification.warning(content)
  },
  // 确认窗体
  confirm(content, options = {}) {
    return MessageBox.confirm(content, '提示', {
      confirmButtonText: '确定',
      cancelButtonText: '取消',
      type: 'warning',
      ...options
    })
  },
  // 提交内容
  prompt(content) {
    return MessageBox.prompt(content, '提示', {
      confirmButtonText: '确定',
      cancelButtonText: '取消',
      type: 'warning'
    })
  },
  // 打开遮罩层
  loading(content) {
    loadingInstance = Loading.service({
      lock: true,
      text: content,
      spinner: 'el-icon-loading',
      background: 'rgba(0, 0, 0, 0.7)'
    })
  },
  // 关闭遮罩层
  closeLoading() {
    loadingInstance.close()
  },
  // 富文本预览
  previewRichText(content) {
    MessageBox.alert(`<div style="max-height: 80vh;overflow: auto">${content}</div>`, '预览', {
      dangerouslyUseHTMLString: true,
      customClass:"self-preview-model"
    })
  },
  // 视频预览
  showVideo(title = '视频预览', url) {
    MessageBox({
      customClass: 'preview-modal',
      title,
      message: `
    <video
            style="object-fit:cover;"
            src="${url}"
            autoplay
            controls="controls">
             升级浏览器！
          </video>
    `,
      dangerouslyUseHTMLString: true,
      showConfirmButton: false,
      width: 600,
      showClose: true,
      callback(action, instance) {
        if (action === 'cancel') {
          const vidoe = instance.$el.querySelector('video')
          vidoe.src = null
        }
      }
    })
  }
}
