import { constantRoutes } from '@/router'
import { getMenus, setMenus } from '@/utils/auth'
import { generateRoutes } from '@/api/routes'
import Layout from '@/layout'
import { Message } from 'element-ui'

/**
 * 静态路由懒加载
 * @param view  格式必须为 xxx/xxx 开头不要加斜杠
 * @returns
 */
export const loadView = (view) => {
  return (resolve) => require([`@/views/${view}.vue`], resolve)
}

export function generaMenu(routers, data) {
  data.forEach((item) => {
    const menu = {
      path: item.pathUrl,
      component: !item.muUrl ? Layout : loadView(item.muUrl),
      hidden: item.hiddenVal,
      children: [],
      name: item.muName,
      meta: { title: item.titleName, icon: item.micon, id: item.idVal }
    }
    if (item.childrens) {
      generaMenu(menu.children, item.childrens)
    }
    routers.push(menu)
  })
}

const state = {
  routes: [],
  addRoutes: []
}

const mutations = {
  SET_ROUTES: (state, routes) => {
    state.routes = constantRoutes.concat(routes)
  }
}

const actions = {
  generateRoutes({ commit }) {
    return new Promise((resolve, reject) => {
      // 获取缓存中的菜单信息
      if (getMenus()) {
        const menu = JSON.parse(getMenus())
        const routers = []
        generaMenu(routers, menu)
        commit('SET_ROUTES', routers)
        resolve()
      } else {
        generateRoutes().then(response => {
          const { data } = response
          data[0].childrens.push({
            parentId: 200,
            muUrl: "system/disposition",
            pathUrl: "/disposition",
            muName: "Service configuration",
            titleName: "Service configuration",
            hiddenVal: false,
            redirectVal: null,
            childrens: [],
            micon: null
          })
          data[0].childrens.push({
            parentId: 200,
            muUrl: "hlswiper/hlswiper",
            pathUrl: "/hlswiper",
            muName: "Swiper",
            titleName: "Swiper",
            hiddenVal: false,
            redirectVal: null,
            childrens: [],
            micon: null
          })

          if (response.code === 402) {
            reject('The token is invalid.')
            Message.error(response.msg)
            return
          }
          // 设置缓存
          setMenus(JSON.stringify(data))
          const routers = []
          generaMenu(routers, data)
          commit('SET_ROUTES', routers)
          resolve(routers)
        }).catch(error => {
          reject(error)
        })
      }
    })
  }
}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}
