/**
 * 名称: Vue-ppvod-upload
 * 作者：大图计算机网络工作室 (www.datll.com) QQ:834023388 承接tp5、tp6框架前端，Vuejs项目，模板仿站【电影站、视频站、导航站、H5网站】
 * 时间：2020-10-10
 */
import axios from 'axios'
import SparkMD5 from 'spark-md5'
import md5 from 'js-md5'
import { dvoUpload } from '@/api/upload'
import modal from '@/plugins/modal'

var uploadindexloaded = 0
export default {
  cache: {},
  data: {
    is_async: true, // 是否异步上传
    debug: true, // 是否开启调试模式
    files: [],
    chunkSize: 10 * 1024 * 1024, // 每片文件大小为5M
    mimeTypes: '.ass,.srt,.rmvb,.flv,.vob,.mp4,.mov,.3gp,.wmv,.mp3,.mkv,.mpg,.ts,.avi,.mpeg,.avi,.rm,.wav,.asf,.divx,.mpg,.mpe,.vod,.mts',
    userinfo: {
      userId: 'ABDBWXKJLKJ',
      category: ''
    }
  },
  /**
   * 初始化
   * @param {*} params
   * @return {Object} this
   **/
  create(params) {
    this.data = Object.assign(this.data, params)
    this.CancelToken = axios.CancelToken
    this.http = dvoUpload
    this.log('初始化成功')
    return this
  },
  error(error) {
    if (error.response) {
      if (error.response.status === 403) {

      }
      if (error.response.status === 401) {

      }
    }
    return Promise.reject(error)
  },
  /**
   * 添加文件
   * @param {*} file 文件列表
   */
  async add(file) {
    console.log(file.type)
    this.log('有文件进来了')
    // 绑定一个文件添加完成事件
    const ext = '.' + file.type.split('/')[1]
    // 验证文件类型
    if (this.data.mimeTypes.indexOf(ext) === -1) {
      modal.msgError('文件类型不支持')
      throw { code: -1, msg: '文件类型不支持' }
    }
    const fileMd5 = await this.getFileMd5(file)
    console.log(fileMd5, this.data.files.length)
    // 验证文件是否已经再上传列表中
    if (this.data.files.findIndex(item => item.md5 === fileMd5 && item.status == 'uploading') !== -1) {
      // 跳过 循环
      throw { code: -1, msg: '文件已经在上传列表中' }
    }
    // 获取文件封面
    const fileCover = await this.getFileCover(file)
    // const fileCover = '';
    var fileblock = {
      fileChunks: this.slicer(file),
      blockNum: Math.ceil(file.size / this.data.chunkSize), // 分片数量
      upchunk: 0, // 已上传分片数量
      uploaded: 0, // 已上传 字节
      cover: fileCover,
      status: 'ready',// 上传状态
      chunkprogress: { byte: 0, percent: 0 }, // 分片上传进度
      name: file.name,
      size: file.size,
      type: file.type,
      lastModifiedDate: file.lastModifiedDate,
      md5: fileMd5,
      uniqueFileName: md5('' + this.data.userinfo.userId + file.name + file.type + file.lastModifiedDate + file.size),
      cancel: null,
      onProgress: file.onProgress ? file.onProgress : null
    }
    this.data.files.push(fileblock)
    this.log('文件切片成功')
    return true
  },
  /**
   * 上传指定文件
   * @param  index 文件索引
   */
  async oneUpload(key, _file) {
    const file = _file ? _file : this.data.files[key]
    console.log(this.data.files)

    this.data.uploadindex = key
    file.status = 'uploading'
    this.out('onUpload', file)
    var uploadCheck
    uploadCheck = await this.beforeSendFile(file)
    if (uploadCheck.skip == false) {
      this.log('文件已经上传过了', uploadCheck)
      file.chunkprogress = {
        Total: file.size,
        byte: file.size,
        surplus: 0,
        percent: 100,
        chunkbyte: this.data.chunkSize,
        chunk: file.blockNum,
        blockNum: file.blockNum,
        speed: 0 // 后面想办法计算
      }
      file.status = 'success'
      this.out('OnFileUploadSuccess', { info: uploadCheck, index: key })
      this.out('OnFileUploadProgress', {
        files: this.data.files,
        progressEvent: file.chunkprogress,
        chunk: file.blockNum
      })
      return uploadCheck
    } else {
      var chunk = await this.chunksupload(file)
      if (chunk) {
        this.log(chunk, 'chunk')
        var info = await this.afterSendFile(file)
        this.log('OnFileUploadSuccess')
        file.status = 'success'
        return info
      } else {
        this.log('文件上传失败')
        return false
      }
    }
  },
  async upload(file) {
    await this.add(file)
    return await this.oneUpload(this.data.files.length - 1)
  },
  /**
   * 上传所有文件
   * @param {*} files 文件列表
   */
  async allUpload(files) {
    if (!files) {
      files = this.data.files
    }
    if (this.data.is_async) {
      for (let i = 0; i < files.length; i++) {
        if (files[i].status === 'pause') {
          this.data.files[i].status = 'uploading'
        }
      }
      this.runInParallel(files.length)
    } else {
      this.syncupload(files)
    }
  },
  // 异步并发 此方法不稳定
  async runInParallel(len) {
    this.log('异步并发上传')
    // 创建一个数组，其中包含与所需函数数相对应的元素数
    const iterations = [...Array(len).keys()]
    var files = this.data.files
    return await Promise.all(
      iterations.map(async i => {
        try {
          // 执行异步操作
          if (files[i].status === 'uploading' || files[i].status === 'success') {
            // 跳过正在上传的文件
          } else {
            this.oneUpload(i)
          }
          await scrape()
        } catch (e) {
          // 句柄错误
        } finally {
          // 关闭页面和浏览器
        }
      })
    )
  },
  //同步上传
  async syncupload(files) {
    this.log('开始同步上传...')
    for (let i = 0; i < files.length; i++) {
      if (files[i].status === 'uploading' || files[i].status === 'success') {
        continue
      }
      if (files[i].status === 'pause') {
        files[i].status = 'ready'
      }
      // this.data.uploadindex = i;
      // this.data.files[i].status = 'uploading';
      this.oneUpload(i, files[i])
    }
  },
  /**
   * 上传分片
   * @param {*} fileChunks
   * @param {*} md5
   * @returns 逻辑判断
   */
  async chunksupload(file) {
    // 开始计时
    const { fileChunks, md5 } = file
    for (let i = 0; i < fileChunks.length; i++) {
      const blockNum = file.blockNum
      const chunk = fileChunks[i]
      var ChunksCheck = await this.beforeSend({
        id: chunk.id,
        chunk: chunk.chunk,
        uniqueFileName: chunk.uniqueFileName,
        size: chunk.blobSize
      })

      if (file.status == 'pause') {
        // 停止
        console.log('已经暂停')
        return 'pause'
      }
      if (!ChunksCheck) {
        this.log(ChunksCheck, '片段已经上传过了')
        file.chunkprogress.byte += chunk.blobSize
        let progress = file.chunkprogress.byte
        file.chunkprogress = {
          ...file.chunkprogress,
          Total: chunk.filesize,
          surplus: (chunk.filesize - chunk.blobSize),
          percent: Math.round((progress / chunk.filesize) * 100),
          chunkbyte: chunk.blobSize,
          chunk: chunk.chunk,
          blockNum: blockNum,
          speed: chunk.blobSize // 后面想办法计算
        }
      } else {
        var formData = new FormData()
        var param = {
          'md5': md5,
          'uniqueFileName': chunk.uniqueFileName,
          'chunk': chunk.chunk,
          'chunks': chunk.chunks,
          'id': chunk.id,
          'name': chunk.name,
          'type': chunk.type,
          'lastModifiedDate': chunk.lastModifiedDate,
          'size': file.size,
          'file': chunk.blob
        }
        // 插入附加参数 userinfo
        Object.keys(this.data.userinfo).forEach(key => formData.append(key, this.data.userinfo[key]))
        Object.keys(param).forEach(key => formData.append(key, param[key]))
        // 缓存单个切片的字节进度
        var progressloaded = 0
        await this.http.post(formData, {
          headers: this.data.headers ? this.data.headers : {
            'Accept': '*/*',
            'Content-Type': 'multipart/form-data,'
          },
          onUploadProgress: (progressEvent) => {
            // 这个是走了多少
            let loaded = (chunk.blobSize * (progressEvent.loaded / progressEvent.total))
            file.chunkprogress.byte += loaded - progressloaded
            let progress = file.chunkprogress.byte
            let percent = Math.round((progress / chunk.filesize) * 100)
            file.chunkprogress = {
              Total: chunk.filesize,
              byte: Math.round(progress),
              surplus: chunk.filesize - progress,
              percent: percent > 100 ? 100 : percent,
              chunkbyte: chunk.blobSize,
              chunk: chunk.chunk,
              blockNum: blockNum,
              speed: loaded
            }
            const onProgress = file.onProgress
            onProgress && onProgress(percent > 100 ? 100 : percent)
            this.out('OnFileUploadProgress', {
              files: this.data.files,
              progressEvent: progressEvent,
              chunk: chunk.chunk
            })
            progressloaded = loaded
          },
          cancelToken: new this.CancelToken(function executor(c) {
            file.cancel = c
          })
        })
      }
      // 已完成的分片
      file.upchunk = i
      if (i == (fileChunks.length - 1)) {
        return true
      }
    }
  },

  /**
   * 上传文件前的验证
   * @param {Object} file
   * @returns {Promise}
   * @description  上传文件前的验证，如果文件已经上传过了，返回false 并获得文件信息
   * */
  beforeSendFile(file) {
    return new Promise((resolve, reject) => {
      let data = {
        status: 'md5Check',
        md5: file.md5,
        name: file.name.replace('.' + file.ext, ''),
        _: new Date().getTime()
      }
      this.http.get(data).then(res => {
        //存在
        if (res.data != false) {
          res.skip = false  // 跳过
          resolve(res)
          return
        } else {
          resolve({
            skip: true
          })
        }
      }).catch(e => reject(e))
    })
  },

  /**
   * 上传分片之前的验证器
   * @param {any} block
   * @returns {Promise}
   * @description  用于验证分片是否已经上传过，如果已经上传过则跳过，否则上传
   * */
  beforeSend(block) {
    var that = this
    return new Promise((resolve, reject) => {
      this.http.get({
        status: 'chunkCheck',
        name: block.uniqueFileName,
        chunkIndex: block.chunk,
        size: block.size,
        _: new Date().getTime()
      }).then((res) => {
          if (res.data !== false) {
            //若存在就跳过
            resolve(false)
          } else {
            resolve(true)
          }
        }, function(jqXHR, textStatus, errorThrown) {
          //任何形式的验证失败，都触发重新上传
          resolve(true)
        }
      ).catch(err => {
        this.out('error', err)
        resolve(true)
      })
    })
  },

  /**
   * 上传完成所有分片后执行
   * @param file
   * @returns {Promise}
   * @description 上传完成所有分片后执行,用于合并分片请求
   * */
  afterSendFile(file) {
    console.log('afterSendFile')
    var that = this
    return new Promise((resolve, reject) => {
      var chunksTotal = 1
      chunksTotal = Math.ceil(file.size / that.chunkSize)
      var ext = file.name.split('.').pop()
      this.http.get({
        status: 'chunksMerge',
        category: '',
        tmpl: '',
        name: file.uniqueFileName,
        chunks: file.blockNum,
        ext: ext,
        fileoldname: file.name.replace('.' + ext, ''),
        md5: file.md5,
        _: new Date().getTime()
      }).then((res) => {
        resolve(res)
      }).catch(err => {
        console.log(err)
        reject(err)
      }).catch(err => {
        this.out('error', err)
        resolve(err)
      })

    })
  },

  /**
   * 文件切片
   * @param {Object} file 文件对象
   * @return {Array} fileChunks 文件切片数组
   */
  slicer(file, index) {
    var fileChunks = [] //保存所有的文件切片数据
    var fileSize = file.size
    var fileName = file.name //文件原名称
    var currentSize = 0  //当前文件切片之前已经切过的大小
    var blockNum = Math.ceil(file.size / this.data.chunkSize)
    for (var i = 0; i <= blockNum; i++) {
      var blobSize = Math.min(this.data.chunkSize, fileSize - currentSize)
      var blob = file.slice(currentSize, currentSize + blobSize)
      var block = {
        blob: blob,
        chunk: i,
        chunks: blockNum,
        id: 'WU_FILE_0',
        blobSize: blobSize,
        start: currentSize,
        end: (currentSize + blobSize),
        name: fileName,
        type: file.type,
        lastModifiedDate: file.lastModifiedDate,
        filesize: file.size,
        uniqueFileName: md5('' + this.data.userinfo.userId + file.name + file.type + file.lastModifiedDate + file.size)
      }
      currentSize += blobSize
      if (blobSize == 0) break
      fileChunks.push(block)
      this.out('OnChunkSlicer', { chunk: block, id: i, fileindex: index })
    }
    return fileChunks
  },
  /**
   * 提取视频封面
   * */
  getFileCover(file) {
    return new Promise((resolve) => {

      // 根据文件大小获取 currentTime 时间
      var currentTime = 3
      if (file.size < 1024 * 1024 * 10) {
        currentTime = 5
      } else if (file.size < 1024 * 1024 * 100) {
        currentTime = 10
      } else if (file.size < 1024 * 1024 * 500) {
        currentTime = 20
      } else if (file.size < 1024 * 1024 * 1000) {
        currentTime = 30
      } else if (file.size < 1024 * 1024 * 2000) {
        currentTime = 60
      }
      var blob = new Blob([file]), url = URL.createObjectURL(blob), video = document.createElement('video')
      video.preload = 'metadata', video.currentTime = 5, video.src = url
      video.onloadedmetadata = function() {
        setTimeout(() => {
          window.URL.revokeObjectURL(video.src)
          var canvas = document.createElement('canvas')
          canvas.width = video.videoWidth
          canvas.height = video.videoHeight
          canvas.getContext('2d').drawImage(video, 0, 0, canvas.width, canvas.height)
          var img = canvas.toDataURL('image/jpeg')
          resolve(img)
          canvas = null
          video = null
        }, 100)
      }
    })
  },
  /**
   * 获取文件md5
   * @param {Object} file 文件对象
   * @return {Promise<String>} md5 文件md5
   * @description 用于断点续传,依赖spark-md5.js 安装命令 npm install spark-md5 --save
   */
  getFileMd5(file) {
    this.log('获取文件MD5...')
    let fileReader = new FileReader()
    // fileReader.readAsBinaryString(file)
    fileReader.readAsBinaryString(file.slice(0, 10 * 1024 * 1024))
    let spark = new SparkMD5()
    return new Promise((resolve) => {
      fileReader.onload = (e) => {
        spark.appendBinary(e.target.result)
        resolve(spark.end())
      }
    })
  },

  /**
   * 事件注册器
   * @param {String} name 事件名称
   * @param {Function} callback 回调函数
   * @description 事件名称有 仅支持以下事件
   */
  on(name, callback) {
    var fun = ['OnFileAddStart', 'OnFileAddEnd', 'onProgress', 'onUpload', 'OnFileUploadSuccess', 'OnFileUploadError', 'OnRequest', 'OnResponse', 'OnChunkSlicer', 'error']
    if (fun.indexOf(name) === -1) {
      return
    }
    this.event[name] = callback
  },

  /**
   * 触发事件器
   * @param {String} fun 事件名称
   * @param {Object} params 回调参数
   * @description 事件名称有 仅支持以下事件
   */
  out(fun, params) {
    if (typeof this.event[fun] === 'function') {
      this.event[fun](params)
    }
  },
  log(...e) {
    if (this.data.debug) {
      console.log(...e)
    }
  },
  // 删除上传文件
  deleteFile(index, type) {
    if (type == 'all') {
      this.data.files = []
    } else {
      this.data.files.splice(index, 1)
    }
  },
  // 操作
  operateFile(index, type) {
    // 暂停
    if (type == 'pause') {
      this.data.files[index].status = 'pause'
      this.data.files[index].xhr.abort()
    }
    // 继续
    if (type == 'continue') {
      this.data.files[index].status = 'uploading'
      this.uploadFile(this.data.files[index])
    }
    // 重试
    if (type == 'retry') {
      this.data.files[index].status = 'uploading'
      this.uploadFile(this.data.files[index])
    }
  },
  /**
   * 事件容器
   * @description
   */
  event: {},

  pauseUpload(index) {
    this.data.files[index].status = 'pause'
    this.log('开始暂停...')
    this.data.files[index].cancel()
  }
}

